/*
 BH1750.c
 * Software to accompany the article -
 * "The Quark D2000 Development Board: I2C Interfacing"
 * Interface a BH1750 light sensor module to the
 * D2000 board
 *****************************************************
 ** This software is offered strictly as is with no **
 ** warranties whatsoever. Use it at your own risk. **
 *****************************************************
 */

#include "qm_common.h"
#include "qm_i2c.h"
#include "qm_pinmux.h"

#define BYTES2READ (2)
#define BH1750addr (0x23) 	/* slave I2c address */

/* globals */
qm_i2c_config_t I2Ccfg; 	/* I2C structure variables   */
uint8_t BHlxdata[2]; 		/* data read from the sensor */
uint16_t lux; 				/* converted lux value       */
qm_i2c_status_t I2Cstatus; 	/* see i2c.h for values      */

/*__________________________________________________________________________*/
int wireBegin() {
	/*  Enable I2C 0 */
	clk_periph_enable(CLK_PERIPH_CLK | CLK_PERIPH_I2C_M0_REGISTER);
	/* set IO pins for SDA and SCL */
	qm_pmux_select(QM_PIN_ID_6, QM_PMUX_FN_2);
	qm_pmux_select(QM_PIN_ID_7, QM_PMUX_FN_2);

	/* Configure I2C */
	I2Ccfg.address_mode = QM_I2C_7_BIT;
	I2Ccfg.mode = QM_I2C_MASTER;
	I2Ccfg.speed = QM_I2C_SPEED_STD;

	/* set the configuration through the structure and return if failure */
	if (qm_i2c_set_config(QM_I2C_0, &I2Ccfg)) {
		QM_PUTS("Error: I2C_0 config\n");
		return (errno);
	} else {
		return (0);
	}
}
/*___________________________________________________________________________*/

int initBH1750() {
	/* configure the sensor mode */

	uint8_t BHmodedata[1] = {0x10}; /* BH1750 initialization code for 1 lux resolution */
	if (qm_i2c_master_write(QM_I2C_0, BH1750addr, BHmodedata,
			sizeof(BHmodedata), true, &I2Cstatus)) {
		return (errno); 	/* error code (errn) */
	}
	clk_sys_udelay(180000); /* 180 ms delay as per BH1750 data sheet */
	return (0);
}
/*____________________________________________________________________________*/
int main(void) {

	int rescode;

	QM_PUTS("BH1750 Sensor Test\n");
	/* configure I2C and quit if error */
	if ( (rescode=wireBegin() ) ) {
		QM_PRINTF("Error: Could not configure I2C (errno=%d)\n",rescode);
		return (1);
	}
	/* write sensor mode and quit if error */
	if ( (rescode=initBH1750())) {
		QM_PRINTF("Error: Could not write BH1750 mode (errno=%d)\n",rescode);
		return (1);
	}
	/* read sensor and display result loop*/
	while (1) {
		/* read the sensor data */
		rescode=qm_i2c_master_read(QM_I2C_0, BH1750addr, BHlxdata, BYTES2READ, true,
				&I2Cstatus);
		if(rescode) {
			QM_PRINTF("Error: Could not read BH1750 (errno=%d)\n",rescode);
			return (1);
		}
		/* translate the data */
		lux = ((BHlxdata[0] << 8) | BHlxdata[1]) / 1.2;
		QM_PRINTF("%d lux\n",lux);
		/* wait 120 ms  before the next read as per BH1750 data sheet */
		clk_sys_udelay(120000);
		/* delay 1.5 sec for the loop print - change as desired */
		clk_sys_udelay(1500000);
	}
	/* never gets here */
	return (0);
}
