/*
  BH1750.ino
   Software to accompany the article -
   "The Quark D2000 Development Board: I2C Interfacing"
   Interface a BH1750 light sensor module to the
   Arduino UNO board
 *****************************************************
 ** This software is offered strictly as is with no **
 ** warranties whatsoever. Use it at your own risk. **
 *****************************************************
*/

#include <Wire.h>

#define BYTES2READ (2)
#define BH1750addr (0x23)   /* slave I2c address */

/* globals */
int status;

void setup() {
  Wire.begin();
  Serial.begin(9600);
  Serial.println("BH1750 sensor test");
  status = initBH1750();
  if (status) {
    Serial.print("Error ");
    Serial.print(status);
    Serial.println("Could not write BH1750 mode");
  }
}

void loop() {
  byte BHlxdata[2];    /* data read from the sensor */
  unsigned int lux;    /* converted lux value       */

  /* read the sensor data */
  Wire.requestFrom(BH1750addr,BYTES2READ);
  BHlxdata[0] = (byte)Wire.read();
  BHlxdata[1] = (byte)Wire.read();
  /* translate the data */
  lux = ( (BHlxdata[0] << 8) | BHlxdata[1]) / 1.2;
  Serial.print(lux);
  Serial.println(" lux");
  /* wait 120 ms  before the next read as per BH1750 data sheet */
  delay(120);
  /* delay for the loop print - change as desired */
  delay(1500);
}

int initBH1750()
{
  byte BHmodedata = 0x10; /* BH1750 initialization code for 1 lux resolution */

  Wire.beginTransmission(BH1750addr);
  Wire.write(BHmodedata);
  if ( (status = Wire.endTransmission() ) ) {
    Serial.print("Error ");
    Serial.print(status);
    Serial.println("Could not write BH1750 mode");
  }
  delay(180); /* 180 ms delay as per BH1750 data sheet */
  return (status);
}


