/*
 * C0220BiZ_Lib.C
 * Software to accompany the article -
 * "The Quark D2000 Development Board: I2C Interfacing"
 * Routines for the I2C operation of the Newhaven
 * C022BiZ-FSW-FBW-3VM 2 X 20 COG display
 *****************************************************
 ** This software is offered strictly as is with no **
 ** warranties whatsoever. Use it at your own risk. **
 *****************************************************
 */

#include "bsp/drivers/include/clk.h"
#include "bsp/drivers/include/qm_gpio.h"
#include "bsp/drivers/include/qm_i2c.h"
#include "bsp/drivers/include/qm_pinmux.h"
#include "bsp/soc/quark_d2000/include/qm_soc_regs.h"
#include <stdbool.h>
#include <string.h>
#include <sys/_stdint.h>
#include <sys/errno.h>

#include "C0220BiZ_Lib.h"

static qm_i2c_status_t I2Cstatus;   /* see i2c.h for values */

/*___________________________________________________________________________*/
void LCD_reset(void) {

	qm_gpio_clear_pin(QM_GPIO_0, LCDRST);
	clk_sys_udelay(200); /* 200 microseconds delay */
	qm_gpio_set_pin(QM_GPIO_0, LCDRST);
}

int LCD_init() {
	/* note: these array sizes are hard-coded in the I2C calls */
	uint8_t c0ini0[2] = { lcdcmd, 0x38 };
	uint8_t c0ini1[2] = { lcdcmd, 0x39 };
	uint8_t c0ini2[5] = { lcdcmd, 0x14, 0x70, 0x5e, 0x6d };
	uint8_t c0ini3[3] = { lcdcmd, 0x0c, 0x01 };
	uint8_t c0ini4[2] = { lcdcmd, 0x06 };

	/* delay for 40 ms upon power up or reset - assume that situation */
	clk_sys_udelay(40000);
	if (qm_i2c_master_write(QM_I2C_0, LCDADDR, c0ini0, 2, true, &I2Cstatus)) {
		return (errno);
	}
	clk_sys_udelay(10000); /* 10 ms after function set table  */
	if (qm_i2c_master_write(QM_I2C_0, LCDADDR, c0ini1, 2, true, &I2Cstatus)) {
		return (errno);
	}
	clk_sys_udelay(10000); /* 10 ms after function set table  */
	if (qm_i2c_master_write(QM_I2C_0, LCDADDR, c0ini2, 5, true, &I2Cstatus)) {
		return (errno);
	}
	clk_sys_udelay(300000); /* 300 ms after follower control   */
	if (qm_i2c_master_write(QM_I2C_0, LCDADDR, c0ini3, 3, true, &I2Cstatus)) {
		return (errno);
	}
	clk_sys_udelay(2000); /* 2 ms delay after clear screen */
	if (qm_i2c_master_write(QM_I2C_0, LCDADDR, c0ini4, 2, true, &I2Cstatus)) {
		return (errno);
	}
	clk_sys_udelay(2000); /* 2 ms delay  */
	return (0);

}

int LCD_clr(void) {
	uint8_t cmdclr[2] = { lcdcmd, 0x01 };
	if (qm_i2c_master_write(QM_I2C_0, LCDADDR, cmdclr, 2, true, &I2Cstatus)) {
		return (errno);
	}
	clk_sys_udelay(2000);
	return (0);
}


int LCD_home(void) {
	uint8_t cmdhome[2] = { lcdcmd, 0x02 };
	if (qm_i2c_master_write(QM_I2C_0, LCDADDR, cmdhome, 2, true, &I2Cstatus)) {
		return (errno);
	}
	clk_sys_udelay(2000);
	return (0);
}


int LCD_display(uint8_t arg) {
	int errorcode = 0;
	uint8_t display[2] = { lcdcmd, 0x0c }; /* display on */

	switch (arg) {
	case 1: {
		errorcode = qm_i2c_master_write(QM_I2C_0, LCDADDR, display, 2, true,
				&I2Cstatus);
		break;
	}
	case 0: {
		display[1] &= 0x08; /* reset bits 0 1 2 */
		errorcode = qm_i2c_master_write(QM_I2C_0, LCDADDR, display, 2, true,
				&I2Cstatus);
		break;
	}
	default: {
		break;
	}
	}
	clk_sys_udelay(2000);
	return (errorcode);

}

int LCD_cursor(uint8_t arg) {
	uint8_t cursor[2] = { lcdcmd, 0x0c }; /* display on cursor off */
	int errorcode = 0;

	switch (arg) {
	case 0: {
		errorcode = qm_i2c_master_write(QM_I2C_0, LCDADDR, cursor, 2, true,
				&I2Cstatus);
		break;
	}
	case 1: {
		cursor[1] = 0x0e;
		errorcode = qm_i2c_master_write(QM_I2C_0, LCDADDR, cursor, 2, true,
				&I2Cstatus);
		break;
	}
	case 2: {
		cursor[1] = 0x0f;
		errorcode = qm_i2c_master_write(QM_I2C_0, LCDADDR, cursor, 2, true,
				&I2Cstatus);
		break;
	}
	case 3: {
		cursor[1] = 0x0d;
		errorcode = qm_i2c_master_write(QM_I2C_0, LCDADDR, cursor, 2, true,
				&I2Cstatus);
		break;
	}
	default: {
		break;
	}
	}
	clk_sys_udelay(2000);
	return (errorcode);
}


int LCD_write(const void * message) {
	uint8_t dta[1] = { lcddta };

	/* note false option in next line */
	if (qm_i2c_master_write(QM_I2C_0, LCDADDR, dta, 1, false, &I2Cstatus)) {
		return (errno);
	}
	if (qm_i2c_master_write(QM_I2C_0, LCDADDR, message, strlen(message), true,
			&I2Cstatus)) {
		return (errno);
	}
	return (0);
}


int LCD_gotoxy(uint8_t x, uint8_t y) {
	uint8_t cmdddram[2] = { lcdcmd, 0 };

	if ( (y > 1)  || (x > 19) ) {
		return (-999);
	}
	y *= 0x40;
	y += x;
	cmdddram[1] = 0x80 | y;
	if (qm_i2c_master_write(QM_I2C_0, LCDADDR, cmdddram, 2, true, &I2Cstatus)) {
		return (errno);
	}
	return (0);
}


int LCD_writexy(uint8_t x, uint8_t y, const void * message) {
	uint8_t cmdddram[2] = { lcdcmd, 0 };
	uint8_t dta[1] = { lcddta };

	if ( (y > 1)  || (x > 19) ) {
		return (-999);
	}
	y *= 0x40;
	y += x;
	cmdddram[1] = 0x80 | y;
	if (qm_i2c_master_write(QM_I2C_0, LCDADDR, cmdddram, 2, true, &I2Cstatus)) {
		return (errno);
	}
	/* note false option in next line */
	if (qm_i2c_master_write(QM_I2C_0, LCDADDR, dta, 1, false, &I2Cstatus)) {
		return (errno);
	}
	if (qm_i2c_master_write(QM_I2C_0, LCDADDR, message, strlen(message), true,
			&I2Cstatus)) {
		return (errno);
	}
	return (0);

}


int LCD_contrast(uint8_t arg) {
	uint8_t cmdcon[2] = { lcdcmd, 0x70 };
	uint8_t cmdIT1[2] = { lcdcmd, 0x39 };

	if (arg > 15) {
		return (-999);
	}
	if (qm_i2c_master_write(QM_I2C_0, LCDADDR, cmdIT1, 2, true, &I2Cstatus)) {
		return (errno);
	}
	clk_sys_udelay(10000); /* 10 ms after function set table  */
	cmdcon[1] |= arg;
	if (qm_i2c_master_write(QM_I2C_0, LCDADDR, cmdcon, 2, true, &I2Cstatus)) {
		return (errno);
	}
	return (0);
}

int LCD_ICC(uint8_t addr, uint8_t * chrcode, uint8_t length) {
	/* note: these array sizes are hard coded */
	uint8_t cmdIT0[2] = { lcdcmd, 0x38 };
	uint8_t cmdIT1[2] = { lcdcmd, 0x39 };
	uint8_t dta[1] = { lcddta };
	uint8_t cmdCRAM[2] = { lcdcmd, 0x40 };

	/* check for address out of bounds */
	if(addr>7)
	{
		return (-999);
	}
	/* set instruction table 0 */
	if (qm_i2c_master_write(QM_I2C_0, LCDADDR, cmdIT0, 2, true, &I2Cstatus)) {
		return (errno);
	}
	clk_sys_udelay(10000); /* 10 ms after function set table  */

	/* set CGRAM address to write char data to - CGRAM starts at 0x40 */
	cmdCRAM[1] = (addr * 8) + 0x40;
	if (qm_i2c_master_write(QM_I2C_0, LCDADDR, cmdCRAM, 2, true, &I2Cstatus)) {
		return (errno);
	}

	/* write the pattern data
	* note false option in next line
	*/
	if (qm_i2c_master_write(QM_I2C_0, LCDADDR, dta, 1, false, &I2Cstatus)) {
		return (errno);
	}
	if (qm_i2c_master_write(QM_I2C_0, LCDADDR, chrcode, length, true,
			&I2Cstatus)) {
		return (errno);
	}
	/* reset to our default Table 1 */
	if (qm_i2c_master_write(QM_I2C_0, LCDADDR, cmdIT1, 2, true, &I2Cstatus)) {
		return (errno);
	}
	clk_sys_udelay(10000); /* 10 ms after function set table  */
	return (0);
}

int LCD_writedat(uint8_t datum) {
	uint8_t dta[2] = { lcddta, 0 };

	dta[1] = datum;
	if (qm_i2c_master_write(QM_I2C_0, LCDADDR, dta, 2, true, &I2Cstatus)) {
		return (errno);
	}
	return (0);
}

int LCD_writedatxy(uint8_t x, uint8_t y, uint8_t datum)
{
	uint8_t cmdddram[2] = { lcdcmd, 0x00 };
	uint8_t dta[2] = { lcddta, 0x00 };

	if ( (y > 1)  || (x > 19) ) {
		return (-999);
	}
	y *= 0x40;
	y += x;
	cmdddram[1] = 0x80 | y;
	dta[1]=datum;
	if (qm_i2c_master_write(QM_I2C_0, LCDADDR, cmdddram, 2, true, &I2Cstatus)) {
		return (errno);
	}

	if (qm_i2c_master_write(QM_I2C_0, LCDADDR, dta, 2, true, &I2Cstatus)) {
		return (errno);
	}
	return (0);

}

/* end */
