/*
 * LCDdemo.c
 * Software to accompany the article -
 * "The Quark D2000 Development Board: I2C Interfacing"
 * Demonstration for the I2C operation of the Newhaven
 * C022BiZ-FSW-FBW-3VM 2 X 20 COG display
 *****************************************************
 ** This software is offered strictly as is with no **
 ** warranties whatsoever. Use it at your own risk. **
 *****************************************************
 */

#include "bsp/drivers/include/clk.h"
#include "bsp/drivers/include/qm_gpio.h"
#include "bsp/drivers/include/qm_i2c.h"
#include "bsp/drivers/include/qm_pinmux.h"

#include "C0220BiZ_Lib.h"

/* Globals */

/* GPIO configuration structure (see qm_gpio.h for details) */
static qm_gpio_port_config_t portcfg;
/*  I2C structure variables  */
static qm_i2c_config_t I2Ccfg;

int main() {

	uint8_t Banner1[] = { "C0220BiZ Library" };
	uint8_t Banner2[] = { "Demonstration" };

	uint8_t SmileyChar[8] = { 0x0, 0xa, 0x0, 0x4, 0x11, 0xe, 0x0 };
	uint8_t FrowneyChar[8] = { 0x0, 0xa, 0x0, 0x4, 0x0, 0xe, 0x11 };
	uint8_t BatChar0[8] = { 0x4, 0x1f, 0x11, 0x11, 0x11, 0x11, 0x1f };
	uint8_t BatChar1[8] = { 0x4, 0x1f, 0x11, 0x11, 0x11, 0x1f, 0x1f };
	uint8_t BatChar2[8] = { 0x4, 0x1f, 0x11, 0x11, 0x1f, 0x1f, 0x1f };
	uint8_t BatChar3[8] = { 0x4, 0x1f, 0x11, 0x1f, 0x1f, 0x1f, 0x1f };
	uint8_t BatChar4[8] = { 0x4, 0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 0x1f };
	uint8_t LockedChar[8] = { 0xe, 0x11, 0x11, 0x1f, 0x1b, 0x1b, 0x1f };

	/* configure GPIO RST bit to output by setting the associated bit in the DDR */
	portcfg.direction = 1 << 5; /* equivalent to portcfg.direction = BIT(LED);*/
	qm_gpio_set_config(QM_GPIO_0, &portcfg);
	qm_gpio_set_pin(QM_GPIO_0, LCDRST); /* set LCDRST high (reset is active low) */
	/*  Enable I2C 0 */
	clk_periph_enable(CLK_PERIPH_CLK | CLK_PERIPH_I2C_M0_REGISTER);
	/* set IO pins for SDA and SCL */
	qm_pmux_select(QM_PIN_ID_6, QM_PMUX_FN_2);
	qm_pmux_select(QM_PIN_ID_7, QM_PMUX_FN_2);
	/* configure I2C */
	I2Ccfg.address_mode = QM_I2C_7_BIT;
	I2Ccfg.mode = QM_I2C_MASTER;
	I2Ccfg.speed = QM_I2C_SPEED_STD;
	if (qm_i2c_set_config(QM_I2C_0, &I2Ccfg)) {
		return 1; /* exit */
	}
	LCD_reset();
	LCD_init();
	while(1){
		LCD_clr();
		/* write opening banner*/
		LCD_writexy(2, 0, Banner1);
		LCD_writexy(3, 1, Banner2);
		clk_sys_udelay(3000000);
		LCD_clr();
		LCD_writexy(1, 0, "Contrast Adjustment");
		LCD_writexy(1, 1, "  Levels 0-15");
		for (uint8_t x = 0; x < 15; x++) {
			LCD_contrast(x);
			clk_sys_udelay(200000);
		}
		for (uint8_t x = 14; x > 0; x--) {
			LCD_contrast(x);
			clk_sys_udelay(200000);
		}
		LCD_clr();
		LCD_writexy(3, 0, "Cursor Control");
		LCD_writexy(6, 1, "off");
		LCD_cursor(0);
		clk_sys_udelay(2000000);
		LCD_writexy(6, 1, " on");
		LCD_cursor(1);
		clk_sys_udelay(2000000);
		LCD_writexy(4, 1, "on blink");
		LCD_cursor(2);
		clk_sys_udelay(2000000);
		LCD_writexy(4, 1, "off blink");
		LCD_cursor(3);
		clk_sys_udelay(2000000);
		LCD_clr();
		LCD_cursor(0);
		LCD_writexy(5, 0, "Write byte");
		LCD_writexy(2, 1, "omega symbol-");
		LCD_writedatxy(15, 1, 0x1e);
		clk_sys_udelay(2000000);
		/* install custom character at CGRAM 0 */
		LCD_ICC(0, BatChar0, 8);
		LCD_ICC(1, BatChar1, 8);
		LCD_ICC(2, BatChar2, 8);
		LCD_ICC(3, BatChar3, 8);
		LCD_ICC(4, BatChar4, 8);
		LCD_ICC(5, SmileyChar, 8);
		LCD_ICC(6, FrowneyChar, 8);
		LCD_ICC(7, LockedChar, 8);
		LCD_clr();
		LCD_writexy(2, 0, "Custom Characters");
		LCD_writedatxy(2, 1, 0);
		LCD_writedatxy(4, 1, 1);
		LCD_writedatxy(6, 1, 2);
		LCD_writedatxy(8, 1, 3);
		LCD_writedatxy(10, 1, 4);
		LCD_writedatxy(12, 1, 5);
		LCD_writedatxy(14, 1, 6);
		LCD_writedatxy(16, 1, 7);
		clk_sys_udelay(5000000);
		/* silly battery animation */
		LCD_clr();
		LCD_writexy(1, 0, "Character Animation");
		LCD_writedatxy(7, 1, 6);
		for (int z = 0; z < 6; z++) {
			LCD_writedatxy(9, 1, 0);
			clk_sys_udelay(250000);
			LCD_writedatxy(9, 1, 1);
			clk_sys_udelay(250000);
			LCD_writedatxy(9, 1, 2);
			clk_sys_udelay(250000);
			LCD_writedatxy(9, 1, 3);
			clk_sys_udelay(250000);
			LCD_writedatxy(9, 1, 4);
			clk_sys_udelay(250000);
		}
	}
	return 0;
}

